package gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import sluzby.SemantickaAnalyzaObsluha;

/**
 * Třída zodpovídající za vykreslování okna programu a jeho funkčnost
 *
 * @author Matthes
 */
public class SemantickaAnalyzaGui extends JFrame
{

    /**
     * Konstantry pro přepínání aktuálně zobrazovaného režimu
     */
    public static final int ZADNA = 0;
    public static final int VEKTOROVA_REPREZENTACE = 1;
    public static final int LSA = 2;
    public static final int PLSA = 3;
    public static final int LDA = 4;
    public static final int VYPOCET = 5;
    public static final int ERROR = 6;

    /**
     * JTable zobrazující hodnoty četnosti slov nebo metod LSA
     */
    private JTable tabMatice;
    /**
     * Obsluha uživatelského rozhraní
     */
    private final SemantickaAnalyzaObsluha obsluha;

    /**
     * Proměnná určující aktuální režim
     */
    private int reprezentace = ZADNA;
    /**
     * Proměnná pro počet vybraných dokumentů
     */
    private int vybranychDokumentu = 0;

    /**
     * Proměnné pro uložení pořadového čísla vybraných dokumentů a nastaveného
     * počtu témat
     */
    private int prvniDokument = 0;
    private int druhyDokument = 0;
    private int pocetTemat = 0;

    /**
     * Přepínač zaokrouhlení
     */
    private boolean zaokrouhli = true;

    /**
     * Konstantní textové řetězce tlačítek apod.
     */
    private static final String PODOBNOST = "Podobnost dokumentů ";
    private static final String NACITANI = "Probíhá výpočet...";
    private static final String TEMAT = "Témat: ";
    private static final String CHYBA = "Soubor nevybrán nebo nastala chyba při načítání.";

    /**
     * tlačítka v okně
     */
    private JButton btVektRep;
    private JButton btSemSVD;
    private JButton btPLSA;
    private JButton btLDA;
    private JButton btTf;
    private JButton btIdf;
    private JButton btIdfJedna;
    private JButton btNactiSoubory;
    private JButton btPlusTemat;
    private JButton btMinusTemat;
    private JButton btTemat;
    private JButton btZaokrouhli;

    /**
     * Prohlížeč souborů
     */
    private JFileChooser fcProchazet;

    /**
     * Panel tlačítek pro výběr témat
     */
    private JPanel pnlTemat;

    /**
     * Popisky v okně
     */
    private JLabel lbPodobnost;
    private JLabel lbNacitani;
    private JLabel lbError;

    /**
     * Konstruktor uživatelského rozhraní
     *
     * @throws HeadlessException
     */
    public SemantickaAnalyzaGui() throws HeadlessException
    {
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setTitle("Sémantická analýza textů");

        obsluha = new SemantickaAnalyzaObsluha();

        initGui();
    }

    /**
     * Metoda inicializace uživatelského rozhraní
     */
    private void initGui()
    {
        btVektRep = new JButton("Vektorová reprezentace");
        btVektRep.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                reprezentace = VYPOCET;
                update();

                obsluha.nastavVektorovouReprezentaci(zaokrouhli);
                reprezentace = VEKTOROVA_REPREZENTACE;
                update();

            }
        });

        btSemSVD = new JButton("LSA");
        btSemSVD.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                reprezentace = VYPOCET;
                update();

                obsluha.nastavSemSVD(zaokrouhli);
                pocetTemat = obsluha.getSoubor().getDokumenty().size();
                btTemat.setText(TEMAT + Integer.toString(pocetTemat));
                reprezentace = LSA;
                update();

            }
        });

        btPLSA = new JButton("PLSA");
        btPLSA.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                reprezentace = VYPOCET;
                update();

                obsluha.nastavPLSA(zaokrouhli);
                pocetTemat = obsluha.getSoubor().getDokumenty().size();
                btTemat.setText(TEMAT + Integer.toString(pocetTemat));
                reprezentace = PLSA;
                update();

            }
        });

        btLDA = new JButton("LDA");
        btLDA.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                reprezentace = VYPOCET;
                update();

                obsluha.nastavLDA(zaokrouhli);
                pocetTemat = obsluha.getSoubor().getDokumenty().size();
                btTemat.setText(TEMAT + Integer.toString(pocetTemat));
                reprezentace = LDA;
                update();

            }
        });

        btTf = new JButton("TF");
        btTf.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                if (!obsluha.getSoubor().isTf())
                {
                    int minRep = reprezentace;
                    reprezentace = VYPOCET;
                    update();

                    obsluha.nastavTf();
                    switch (minRep)
                    {
                        case VEKTOROVA_REPREZENTACE:
                            obsluha.nastavVektorovouReprezentaci(zaokrouhli);
                            break;
                        case LSA:
                            obsluha.nastavSemSVD(pocetTemat, zaokrouhli);
                            break;
                        case PLSA:
                            obsluha.nastavPLSA(pocetTemat, zaokrouhli);
                            break;
                        default:
                            break;
                    }
                    reprezentace = minRep;
                    update();
                }
            }
        });

        btIdf = new JButton("TF - IDF");
        btIdf.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                if (!obsluha.getSoubor().isIdf(false))
                {
                    int minRep = reprezentace;
                    reprezentace = VYPOCET;
                    update();

                    obsluha.nastavTfIDF(false);
                    switch (minRep)
                    {
                        case VEKTOROVA_REPREZENTACE:
                            obsluha.nastavVektorovouReprezentaci(zaokrouhli);
                            break;
                        case LSA:
                            obsluha.nastavSemSVD(pocetTemat, zaokrouhli);
                            break;
                        case PLSA:
                            obsluha.nastavPLSA(pocetTemat, zaokrouhli);
                            break;
                        default:
                            break;
                    }
                    reprezentace = minRep;
                    update();
                }

            }
        });

        btIdfJedna = new JButton("TF - IDF+1");
        btIdfJedna.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                if (!obsluha.getSoubor().isIdf(true))
                {
                    int minRep = reprezentace;
                    reprezentace = VYPOCET;
                    update();

                    obsluha.nastavTfIDF(true);
                    switch (minRep)
                    {
                        case VEKTOROVA_REPREZENTACE:
                            obsluha.nastavVektorovouReprezentaci(zaokrouhli);
                            break;
                        case LSA:
                            obsluha.nastavSemSVD(pocetTemat, zaokrouhli);
                            break;
                        case PLSA:
                            obsluha.nastavPLSA(pocetTemat, zaokrouhli);
                            break;
                        default:
                            break;
                    }
                    reprezentace = minRep;
                    update();
                }
            }
        });

        SemantickaAnalyzaGui gui = this;

        btNactiSoubory = new JButton("Načíst soubory");
        btNactiSoubory.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                reprezentace = VYPOCET;
                update();

                fcProchazet = new JFileChooser();
                fcProchazet.setMultiSelectionEnabled(true);
                fcProchazet.setCurrentDirectory(new File(System.getProperty("user.dir") + "\\\\res"));
                fcProchazet.setDialogTitle("Otevřít soubory");

                int vysledek = fcProchazet.showOpenDialog(gui);
                if (vysledek == 0 && obsluha.nactiSoubor(fcProchazet.getSelectedFiles(), zaokrouhli))
                {
                    reprezentace = VEKTOROVA_REPREZENTACE;
                    update();
                } else
                {
                    reprezentace = ERROR;
                    update();
                }
            }
        });

        btZaokrouhli = new JButton("Nezaokrouhlovat");
        btZaokrouhli.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                zaokrouhli = !zaokrouhli;
                if (zaokrouhli)
                {
                    btZaokrouhli.setText("Nezaokrouhlovat");
                } else
                {
                    btZaokrouhli.setText("Zaokrouhlovat");
                }
            }
        });

        btPlusTemat = new JButton("+");
        btPlusTemat.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                pocetTemat++;
                if (pocetTemat > obsluha.getSoubor().getDokumenty().size())
                {
                    pocetTemat = obsluha.getSoubor().getDokumenty().size();
                }
                btTemat.setText(TEMAT + Integer.toString(pocetTemat));
            }
        });

        btMinusTemat = new JButton("-");
        btMinusTemat.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                pocetTemat--;
                if (pocetTemat <= 2)
                {
                    pocetTemat = 2;
                }
                btTemat.setText(TEMAT + Integer.toString(pocetTemat));
            }
        });

        btTemat = new JButton(TEMAT + Integer.toString(pocetTemat));
        btTemat.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                switch (reprezentace)
                {
                    case LSA:
                        reprezentace = VYPOCET;
                        update();
                        obsluha.nastavSemSVD(pocetTemat, zaokrouhli);
                        reprezentace = LSA;
                        update();
                        break;
                    case PLSA:
                        reprezentace = VYPOCET;
                        update();
                        obsluha.nastavPLSA(pocetTemat, zaokrouhli);
                        btTemat.setText(TEMAT + Integer.toString(pocetTemat));
                        reprezentace = PLSA;
                        update();
                        break;
                    case LDA:
                        reprezentace = VYPOCET;
                        update();
                        obsluha.nastavLDA(pocetTemat, zaokrouhli);
                        btTemat.setText(TEMAT + Integer.toString(pocetTemat));
                        reprezentace = LDA;
                        update();
                        break;
                    default:
                        break;
                }
            }
        });
        pnlTemat = new JPanel(new FlowLayout());
        pnlTemat.add(btMinusTemat);
        pnlTemat.add(btTemat);
        pnlTemat.add(btPlusTemat);

        JPanel pnlReprezentace = new JPanel(new FlowLayout());
        pnlReprezentace.add(btNactiSoubory);
        pnlReprezentace.add(btVektRep);
        pnlReprezentace.add(btSemSVD);
        pnlReprezentace.add(btPLSA);
        pnlReprezentace.add(btLDA);
        pnlReprezentace.add(btZaokrouhli);

        btVektRep.setVisible(false);
        btPLSA.setVisible(false);
        btSemSVD.setVisible(false);

        JPanel pnlVekRep = new JPanel(new FlowLayout());
        pnlVekRep.add(btTf);
        pnlVekRep.add(btIdf);
        pnlVekRep.add(btIdfJedna);
        pnlVekRep.add(pnlTemat);

        lbPodobnost = new JLabel(PODOBNOST + "(klikněte)");
        lbNacitani = new JLabel(NACITANI);
        lbError = new JLabel(CHYBA);

        tabMatice = new JTable(obsluha.getTableModel());
        tabMatice.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        tabMatice.setPreferredScrollableViewportSize(new Dimension(400, 200));
        tabMatice.addMouseListener(new MouseAdapter()
        {
            @Override
            public void mouseClicked(MouseEvent e)
            {
                if (reprezentace != ZADNA)
                {
                    int sloupec = tabMatice.convertColumnIndexToModel(tabMatice.getSelectedColumn());
                    if (reprezentace != VEKTOROVA_REPREZENTACE)
                    {
                        podobnost(sloupec);
                    } else
                    {
                        sloupec--;
                        if (!(sloupec < 0 || sloupec == obsluha.getTableModel().getColumnCount() - 2))
                        {
                            podobnost(sloupec);
                        }
                    }
                }
            }
        });

        JPanel pnlHorni = new JPanel(new BorderLayout());
        pnlHorni.add(pnlReprezentace, BorderLayout.NORTH);
        pnlHorni.add(pnlVekRep, BorderLayout.SOUTH);

        JPanel pnlLabely = new JPanel(new FlowLayout());
        pnlLabely.add(lbNacitani);
        pnlLabely.add(lbError);

        JPanel pnlTabulka = new JPanel(new BorderLayout());
        pnlTabulka.add(new JScrollPane(tabMatice), "Center");
        pnlTabulka.add(pnlLabely, "North");

        getContentPane().setLayout(new BorderLayout());
        getContentPane().add(pnlHorni, "North");
        getContentPane().add(pnlTabulka, "Center");
        getContentPane().add(lbPodobnost, "South");

        pack();
        setSize(800, 600);
        update();
    }

    /**
     * Metoda pro obsluhu zjišťování podobností dokumentů
     *
     * @param sloupec vybraný sloupec v tabulce dat
     */
    private void podobnost(int sloupec)
    {
        switch (vybranychDokumentu)
        {
            case 0:
                prvniDokument = sloupec;
                lbPodobnost.setText(PODOBNOST + obsluha.getSoubor().getNazvyDokumentu()[prvniDokument] + " a (klikněte)");
                vybranychDokumentu++;
                break;
            case 1:
                druhyDokument = sloupec;
                double pod = 0.0;
                switch (reprezentace)
                {
                    case VEKTOROVA_REPREZENTACE:
                        pod = obsluha.getPodobnostVek(prvniDokument, druhyDokument, zaokrouhli);
                        break;
                    case LSA:
                        pod = obsluha.getPodobnostLSA(prvniDokument, druhyDokument, zaokrouhli);
                        break;
                    case PLSA:
                        pod = obsluha.getPodobnostPLSA(prvniDokument, druhyDokument, zaokrouhli);
                        break;
                    case LDA:
                        pod = obsluha.getPodobnostLDA(prvniDokument, druhyDokument, zaokrouhli);
                        break;
                }

                lbPodobnost.setText(PODOBNOST + obsluha.getSoubor().getNazvyDokumentu()[prvniDokument] + " a "
                        + obsluha.getSoubor().getNazvyDokumentu()[druhyDokument] + " je: " + pod + " (pro novou klikněte)");
                vybranychDokumentu++;
                break;
            case 2:
                vybranychDokumentu = 0;
                podobnost(sloupec);
                break;
            default:
                break;
        }

    }

    /**
     * Metoda pro aktualizaci uživatelského rozhraní, která upraví data v
     * tabulce a zobrazená tlačítka
     */
    private void update()
    {
        obsluha.getTableModel().fireTableStructureChanged();
        obsluha.getTableModel().fireTableDataChanged();

        switch (reprezentace)
        {
            case VEKTOROVA_REPREZENTACE:
                btTf.setVisible(true);
                btIdf.setVisible(true);
                btIdfJedna.setVisible(true);
                pnlTemat.setVisible(false);
                btVektRep.setVisible(true);
                btPLSA.setVisible(true);
                btSemSVD.setVisible(true);
                lbNacitani.setVisible(false);
                lbPodobnost.setVisible(true);
                tabMatice.setVisible(true);
                btLDA.setVisible(true);
                lbError.setVisible(false);
                break;
            case PLSA:
            case LSA:
                btTf.setVisible(true);
                btIdf.setVisible(true);
                btIdfJedna.setVisible(true);
                pnlTemat.setVisible(true);
                btVektRep.setVisible(true);
                btPLSA.setVisible(true);
                btSemSVD.setVisible(true);
                lbNacitani.setVisible(false);
                lbPodobnost.setVisible(true);
                tabMatice.setVisible(true);
                btLDA.setVisible(true);
                lbError.setVisible(false);
                break;
            case LDA:
                btTf.setVisible(false);
                btIdf.setVisible(false);
                btIdfJedna.setVisible(false);
                pnlTemat.setVisible(true);
                btVektRep.setVisible(true);
                btPLSA.setVisible(true);
                btSemSVD.setVisible(true);
                lbNacitani.setVisible(false);
                lbPodobnost.setVisible(true);
                tabMatice.setVisible(true);
                btLDA.setVisible(true);
                lbError.setVisible(false);
                break;
            case VYPOCET:
                lbNacitani.setVisible(true);
                tabMatice.setVisible(false);
                lbError.setVisible(false);
                break;
            case ERROR:
                btTf.setVisible(false);
                btIdf.setVisible(false);
                btIdfJedna.setVisible(false);
                pnlTemat.setVisible(false);
                btVektRep.setVisible(false);
                btPLSA.setVisible(false);
                btSemSVD.setVisible(false);
                btLDA.setVisible(false);
                lbPodobnost.setVisible(false);
                lbNacitani.setVisible(false);
                tabMatice.setVisible(false);
                lbError.setVisible(true);
                break;
            case ZADNA:
            default:
                btTf.setVisible(false);
                btIdf.setVisible(false);
                btIdfJedna.setVisible(false);
                pnlTemat.setVisible(false);
                btVektRep.setVisible(false);
                btPLSA.setVisible(false);
                btSemSVD.setVisible(false);
                btLDA.setVisible(false);
                lbPodobnost.setVisible(false);
                lbNacitani.setVisible(false);
                tabMatice.setVisible(true);
                lbError.setVisible(false);
                break;
        }
    }
}
